﻿External Notifier From System to Device
===

librabry can be used in an App Server implementation of Downstream notifications to devices.
Notifications can be sent on the bases of DeviceToken or Topic.

Rest Interface is exposed for testability, however it should always be included as a library.

Prerequisites:
---
FCM account and service account credentials
NB: GCM server admin keys are now deprecated


Usage: 
---
`npm i @anabode/notification-firebase --save`



```javascript
var payload = {
  "Message": "Will be disaplyed in body",
  "Title": "Header",
  "DeviceTopic": ""
};
var conf = {
  fcm: {
    app_name: "APP_NAME",
    scopes: ["https://www.googleapis.com/auth/firebase.messaging"],
    messagingEndpoint: "https://fcm.googleapis.com/v1/projects/",
    databaseURL: "https://APP_NAME.firebaseio.com"

  } 
};

/**
 *  cred - fcm credentials JSON object can be obtained from FCM dasboard for your project [example](https://console.firebase.google.com/u/1)
 */
var cred = {{
  "type": "service_account",
  "project_id": "APP_NAME",
  "private_key_id": "",
  "private_key": "",
  "client_id": "",
  "auth_uri": "",
  "token_uri": "",
  "auth_provider_x509_cert_url": "",
  "client_x509_cert_url": ""
};

  var firebaseWorker = new (require('@anabode/notification-firebase'))(_conf, _cred);

  // callers responsibility of this library to supply it with conf and cred objects
  firebaseWorker.sendToTopic({body: payload}, (e, d) => {
    if (e) {
    console.error(e.message, e.stack);
  } else {
    console.log(d);
  }
```

```javascript
var payload = {
  "Message": "Will be disaplyed in body",
  "Title": "Header",
  "DeviceToken": ""
};
var conf = {
  fcm: {
    app_name: "APP_NAME",
    scopes: ["https://www.googleapis.com/auth/firebase.messaging"],
    messagingEndpoint: "https://fcm.googleapis.com/v1/projects/"
  } 
};

/**
 *  cred - fcm credentials JSON object can be obtained from FCM dasboard for your project [example](https://console.firebase.google.com/u/1)
 */
var cred = {{
  "type": "service_account",
  "project_id": "APP_NAME",
  "private_key_id": "",
  "private_key": "",
  "client_id": "",
  "auth_uri": "",
  "token_uri": "",
  "auth_provider_x509_cert_url": "",
  "client_x509_cert_url": ""
};
firebaseWorker.sendToToken({body: payload}, conf, cred, (e, d) => {
    if (e) {
    console.error(e.message, e.stack);
  } else {
    console.log(d);
  }
```

Alternatively send an array of tokens
```javascript
  var payload = {
    "Message": "Will be disaplyed in body",
    "Title": "Header",
    "DeviceToken": [""]
  };
  var firebaseWorker = new (require('@anabode/notification-firebase'))(_conf, _cred);
  firebaseWorker.sendToMultiToken({body: payload}, (e, d) => {
    if (e) {
      console.error(e.message, e.stack);
    } else {
      console.log(d);
    }
  });
```

Update:

introduced better concurrency by object instantiation

added app.delete() to destroy object since Google keeps HTTP connection alive 

changed to server side GCP admin SDK - which is still beta


TODO:
---
Differentiate between Error/Warning/Info message types

