 'use strict';
var _ = require('lodash');

/**
 * @class
 * @exports
 */
const firebase_model = {
    /**
     * @method
     * @param {Object<input>} input - {body: {
     *  "Message": "_message_",
     * "Title": "_title_",
     * "DeviceToken": "_128_bit_token"
     * }
     * @returns
     */
    topic (input) {
        var _input = input;
        return {
            "message": {
                "topic": _input.body.DeviceTopic,
                "notification": {
                  "body": _input.body.Message,
                  "title": _input.body.Title
                }
            }
        };
    },
     /**
     * @method
     * @param {Object<input>} input - {body: {
     *  "Message": "_message_",
     * "Title": "_title_",
     * "DeviceToken": "_128_bit_token"
     * }
     * @returns
     */
    topic_v2(input) {
        var _input = input;
        return {
            "notification": {
                "body": _input.body.Message,
                "title": _input.body.Title
            }
        };
    },
    /**
     * @method
     * @param {Object<input>} input - {body: {
     *  "Message": "_message_",
     * "Title": "_title_",
     * "DeviceToken": "_128_bit_token"
     * }
     * @returns
     */
    token (input) {
        var _input = input;
        return {
            "message": {
                "token": _input.body.DeviceToken,
                "notification": {
                    "body": _input.body.Message,
                    "title": _input.body.Title
                }
            }
        };
    },
    /**
     * @method
     * @param {Object<input>} input - {body: {
     *  "Message": "_message_",
     * "Title": "_title_",
     * "DeviceToken": "_128_bit_token"
     * }
     * @returns
     */
    token_v2 (input) {
        var _input = input;
        return {
            "token": _input.body.DeviceToken,
            "notification": {
                "body": _input.body.Message,
                "title": _input.body.Title
            }
        };
    },
    /**
     * sends to multiple
     */
    multi_body (input) {
        const _input = input;
        const app_payload = {};
        if (_input.Data) {
            app_payload.data = _input.Data;
        }
        app_payload.notification = {
            body: _input.Message,
            title: _input.Title
        };

        return app_payload;
    },
    /**
     * sends to multiple
     */
    multi_cast_message (input) {
        const _input = input;
        const app_payload = {};
        if (_input.Data) {
            app_payload.data = _input.Data;
        }
        if (!_.isEmpty(_input.droid)) {
            app_payload.android = _input.droid;
        }
        if (!_.isEmpty(_input.ios)) {
            app_payload.apns = _input.ios;
        }
        app_payload.tokens = _.isArray(_input.DeviceToken) ? _input.DeviceToken : [_input.DeviceToken];
        app_payload.notification = {
            body: _input.Message,
            title: _input.Title
        };

        return app_payload;
    },
    /**
     * registrationTokens
     * @returns {String[]}
     */
    multi_tokens (input) {
        const _input = input;
        if (!(_.isArray(_input.DeviceToken))) {
            // throw new Error('Tokens must be array');
            _input.DeviceToken = [_input.DeviceToken];
        }
        return _input.DeviceToken;
    }
}

module.exports = exports = firebase_model;
