'use strict';
const admin = require("firebase-admin"),
    firebaseModel = require('./firebaseModel'),
    _ = require('lodash');

function FirebaseWorker (conf, cred) {
    this.conf = conf;
    this.cred = cred;
}


FirebaseWorker.prototype = {
    /**
     * @deprecated
     * in favour of multi token
     * sends message to single device based on that device\'s token ID
     *  @param {Object<input>} input - { body: {
     *  "Message": "_message_",
     *  "Title": "_title_",
     *  "DeviceToken": "_128_bit_token"
     *  }
     *  @param {Object} conf - required {aws.region: 'us-east-1' }
     *  @param {Object} cred - fcm credentials JSON object see [docs](https://link.com)
     *  @returns {String}
     *
     */
    sendToToken(input, callback) {
        const _this = this;
        const __input = input;
        const app = admin.initializeApp({
            credential: admin.credential.cert(_this.cred),
            databaseURL: _this.conf.fcm.databaseURL
        }, __input.body.DeviceToken);
        app.messaging().send(firebaseModel.token_v2(__input))
            .then((response) => {
                app.delete();
                return callback(null, response);
            })
            .catch((error) => {
                app.delete();
                console.log('Error sending message:', error);
                return callback(error);
            });
    },
    /**
    * sends the same message to many device based on device\'s token ID
    * @param {Object<input>} input - { body: {
    *  "Message": "_message_",
    * "Title": "_title_",
    * "DeviceToken": "_128_bit_token"
    * }
    * @param {Object} conf - required {aws.region: 'us-east-1' }
    * @param {Object} cred - fcm credentials JSON object see [docs](https://link.com)
    * @returns {String}
    */
    async sendToMultiToken(input, callback) {
        const _this = this,
            __input = input;

        let errorHandler = null, dataHandler = null;

        const app = admin.initializeApp({
            credential: admin.credential.cert(_this.cred),
            databaseURL: _this.conf.fcm.databaseURL
        }, _.toString((new Date()).getTime()));

        try {
            // TODO: Limit this to a 100 tokens then break and loop through more
            const response = await app.messaging().sendMulticast(
                firebaseModel.multi_cast_message(__input.body))
                    .catch((e) => {
                        errorHandler = e;
                    })
                if (response.failureCount > 0) {
                    // TODO: implement trigger in logs off the back of this
                    console.error(JSON.stringify(_.remove(_.map(response.responses, 'error'))));
                }
                dataHandler = response;
        } catch (ex) {
            console.log('Error sending message:', JSON.stringify(ex));
            errorHandler = ex;
        } finally {
            await app.delete();
            return callback(errorHandler, dataHandler);
        }
    },

    /**
    * sends message to topic to all devices listening to that topic
    * @method @exports @member sendToTopic
    * @param {Object<input>} input - {body: {
    *  "Message": "_message_",
    * "Title": "_title_",
    * "DeviceTopic": "_128_bit_token"
    * }
    * @param {Object} conf - required {aws.region: 'us-east-1' }
    * @param {Object} cred - fcm credentials JSON object see [docs](https://link.com)
    */
    SendTopic(input, callback) {
        var _this = this;
        var __input = input;
        var firebaseModel = new (require('./firebaseModel'))();
        var app = admin.initializeApp({
            credential: admin.credential.cert(_this.cred),
            databaseURL: _this.conf.fcm.databaseURL
        });

        return app.messaging().sendToTopic(__input.body.DeviceTopic, firebaseModel.topic_v2(__input))
            .then((response) => {
                app.delete();
                return callback(null, response);
            })
            .catch((error) => {
                app.delete();
                console.log('Error sending message:', error);
                return callback(error);
            })
    }
}

module.exports = FirebaseWorker;
